import os
import setuptools
import subprocess

install_requires = [
    'argparse',  # ugh just use >py2.6
    'pip==8.1.2',
    'pex',
    'click',
    'wheel',
]

entry_points = {
    'console_scripts': [
        'setuptize-me = setuptize_me.script:cli'
    ],
}


class Pex(setuptools.Command):
    user_options = []

    def initialize_options(self):
        """Abstract method that is required to be overwritten"""

    def finalize_options(self):
        """Abstract method that is required to be overwritten"""

    def run(self):
        if not os.path.exists('dist/wheel-cache'):
            print('Creating dist/wheel-cache')
            os.makedirs('dist/wheel-cache')

        print('Building wheels')
        subprocess.check_call(['pip', 'wheel', '-w', 'dist/wheel-cache', '.'])

        for entry in entry_points['console_scripts']:
            name, call = tuple([_.strip() for _ in entry.split('=')])
            print('Creating {0} as {1}'.format(name, call))
            pex_cmd = [
              'pex',
              '--no-pypi',
              '--repo=dist/wheel-cache',
              '-o', 'build/bin/{0}'.format(name),
              '-e', call,
              '.',
            ]
            pex_cmd.extend(install_requires)
            print('Running {0}'.format(' '.join(pex_cmd)))
            subprocess.check_call(pex_cmd)


class Venv(setuptools.Command):
    user_options = []

    def initialize_options(self):
        """Abstract method that is required to be overwritten"""

    def finalize_options(self):
        """Abstract method that is required to be overwritten"""

    def run(self):
        venv_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'venv', 'setuptize_me')
        venv_cmd = [
            'virtualenv',
            venv_path
        ]
        print('Creating virtual environment in ', venv_path)
        subprocess.check_call(venv_cmd)
        print('Linking `activate` to top level of project.\n')
        print('To activate, simply run `source activate`.')
        try:
            os.symlink(
                os.path.join(venv_path, 'bin', 'activate'),
                os.path.join(os.path.dirname(os.path.abspath(__file__)), 'activate')
            )
        except OSError:
            print('Unable to create symlink, you may have a stale symlink from a previous invocation.')


setuptools.setup(
    name='setuptize_me',
    author='Loren Carvalho',
    author_email='me@loren.pizza',
    description='bootstrap a python development environment',
    url='https://loren.pizza',
    license='MIT',
    version='0.0.3',
    package_dir={'': 'src'},
    packages=setuptools.find_packages('src'),
    install_requires=install_requires,
    cmdclass={'virtualenv': Venv, 'pexify': Pex},
    entry_points=entry_points,
)
