from string import Template


readme = Template("""
# $project_name
""")

setup_py = Template("""
#!/usr/bin/env python

import os
import setuptools
import subprocess

install_requires = $requirements

entry_points = {
    'console_scripts': []
}

base_path = os.path.dirname(os.path.abspath(__file__))


class Pex(setuptools.Command):
    user_options = []

    def initialize_options(self):
        \"\"\"Abstract method that is required to be overwritten\"\"\"

    def finalize_options(self):
        \"\"\"Abstract method that is required to be overwritten\"\"\"

    def run(self):
        if not os.path.exists(os.path.join(base_path, 'dist/wheel-cache')):
            print('Creating dist/wheel-cache')
            os.makedirs(os.path.join(base_path, 'dist/wheel-cache'))

        print('Building wheels')
        subprocess.check_call(['pip', 'wheel', '-w', 'dist/wheel-cache', '.'])

        for entry in entry_points['console_scripts']:
            name, call = tuple([_.strip() for _ in entry.split('=')])
            print('Creating {0} as {1}'.format(name, call))
            pex_cmd = [
              'pex',
              '--no-pypi',
              '--repo=dist/wheel-cache',
              '-o', 'build/bin/{0}'.format(name),
              '-e', call,
              '.',
            ]
            pex_cmd.extend(install_requires)
            print('Running {0}'.format(' '.join(pex_cmd)))
            subprocess.check_call(pex_cmd)


class Venv(setuptools.Command):
    user_options = [
        ('python-interpreter=', 'p', 'The interpreter to build your venv with')
    ]

    def initialize_options(self):
        if os.environ.get('VIRTUAL_ENV'):
            raise RuntimeError(
                "Please deacivate your current virtual environment before running this subcommand!"
            )
        self.python_interpreter = 'python'

    def finalize_options(self):
        print("Using interpreter: {0}".format(self.python_interpreter))

    def run(self):
        venv_path = os.path.join(base_path, 'venv', '$project_name')
        venv_cmd = [
            'virtualenv',
            '-p', self.python_interpreter,
            venv_path
        ]
        print('Creating virtual environment in {path}'.format(path=venv_path))
        subprocess.check_call(venv_cmd)
        print('Linking `activate` to top level of project.')
        print('To activate, simply run `source activate`.')
        try:
            os.symlink(
                os.path.join(venv_path, 'bin', 'activate'),
                os.path.join(base_path, 'activate'),
            )
        except OSError:
            print('Unable to create symlink, you may have a stale symlink from a previous invocation.')


setuptools.setup(
    name='$project_name',
    author='$author',
    author_email='$email',
    description='$description',
    license='$license',
    version='0.0.1',
    package_dir={'': 'src'},
    packages=setuptools.find_packages('src'),
    install_requires=install_requires,
    entry_points=entry_points,
    cmdclass={'virtualenv': Venv, 'pexify': Pex},
)
""")

mit = Template("""
The MIT License (MIT)

Copyright (c) 2016 $name

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE

""")

setup_cfg = """
[flake8]
ignore = E111,E114,E121,E123,E226,W292
max-line-length = 160
"""

gitignore = """
# Byte-compiled / optimized / DLL files
__pycache__/
*.py[cod]
*$py.class

# C extensions
*.so

# Distribution / packaging
.Python
env/
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
*.egg-info/
.installed.cfg
*.egg

# PyInstaller
#  Usually these files are written by a python script from a template
#  before PyInstaller builds the exe, so as to inject date/other infos into it.
*.manifest
*.spec

# Installer logs
pip-log.txt
pip-delete-this-directory.txt

# Unit test / coverage reports
htmlcov/
.tox/
.coverage
.coverage.*
.cache
nosetests.xml
coverage.xml
*,cover
.hypothesis/

# Translations
*.mo
*.pot

# Django stuff:
*.log
local_settings.py

# Flask stuff:
instance/
.webassets-cache

# Scrapy stuff:
.scrapy

# Sphinx documentation
docs/_build/

# PyBuilder
target/

# IPython Notebook
.ipynb_checkpoints

# pyenv
.python-version

# celery beat schedule file
celerybeat-schedule

# dotenv
.env

# virtualenv
venv/
ENV/

# Spyder project settings
.spyderproject

# Rope project settings
.ropeproject

# Activate symlink
activate
"""
